@@ -7,12 +7,8 @@
 #  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
 from typing import Callable
 
-from .token.abstract import Token
-from .token.attribute import Attribute
-from .token.call import Call
-from .token.literal import Literal
-
-# from parser.selectable import Expression, Global, BuiltIn, Local, Attribute, Literal, ForEach, Template
+from ._token import Token, Getter
+from .token_factory import OnlyPublicGetterTokenFactory, TokenFactory
 
 opening_bracket_to_closing: dict[str, str] = {
     '[': ']',
@@ -37,8 +33,11 @@ def find_first(string: str, condition: Callable, start_position: int = 0, escape
 
 
 class QpelParser:
-    @classmethod
-    def parse(cls, qpel: str) -> Token:
+    def __init__(self, token_factory: TokenFactory | None = OnlyPublicGetterTokenFactory(), allow_spaces: bool = True):
+        self.token_factory = token_factory
+        self.allow_spaces = allow_spaces
+
+    def parse(self, qpel: str) -> Token:
         if not qpel:
             raise ValueError("empty qpel")
 
@@ -51,7 +50,7 @@ class QpelParser:
                 end_quote_index = find_first(qpel, lambda x: x != "'", i + 1)
                 if end_quote_index is None:
                     raise ValueError("missing \"'\"")
-                result = Literal(body=qpel[i+1:end_quote_index])
+                result = self.token_factory.make_literal(body=qpel[i+1:end_quote_index])
                 i = end_quote_index + 1
             elif char.isdigit() or char == '-':
                 first_non_digit_or_dot_index = find_first(qpel, lambda x: x.isdigit() or x in ".-", i + 1)
@@ -60,22 +59,22 @@ class QpelParser:
                 number_str = qpel[i:first_non_digit_or_dot_index]
                 i = first_non_digit_or_dot_index
                 body_type = float if '.' in number_str else int
-                result = Literal(body=body_type(number_str))
+                result = self.token_factory.make_literal(body=body_type(number_str))
             elif result is None and char.isidentifier() or result is not None and char == ".":
                 if char == "." and qpel[i+1] == "[":
                     end = find_paired_bracket(qpel, i + 1)  # TODO: replace with some code aware of all bracket types
-                    args, kwargs = cls._parse_qpel_as_args(qpel[i + 2:end])
+                    args, kwargs = self._parse_qpel_as_args(qpel[i + 2:end])
                     i = end + 1
                     assert not kwargs
                     body = []
                     for arg in args:
                         root = arg.get_root()
-                        if isinstance(root, Attribute):
+                        if isinstance(root, Getter):
                             root.body = result
                             body.append(arg)
                         else:
                             raise TypeError(f"'.[' accepts only attributes, got: {arg}")
-                    result = Literal(body=body)
+                    result = self.token_factory.make_literal(body=body)
                 else:
                     first_non_digit_or_alpha_index = find_first(qpel, lambda x: x.isdigit() or x.isidentifier(), i + 1)
                     if first_non_digit_or_alpha_index is None:
@@ -83,24 +82,27 @@ class QpelParser:
                     start = i if char != '.' else i + 1
                     key_str = qpel[start:first_non_digit_or_alpha_index]
                     i = first_non_digit_or_alpha_index
-                    result = Attribute(key=key_str, body=result)
+                    result = self.token_factory.make_getter(key=key_str, body=result)
             elif char == "[":
                 end = find_paired_bracket(qpel, i)  # TODO: replace with some code aware of all bracket types
-                args, kwargs = cls._parse_qpel_as_args(qpel[i + 1:end])
+                args, kwargs = self._parse_qpel_as_args(qpel[i + 1:end])
                 i = end + 1
                 assert not kwargs
-                result = Literal(body=args)
+                result = self.token_factory.make_literal(body=args)
             elif char == "(":
                 end = find_paired_bracket(qpel, i)  # TODO: replace with some code aware of all bracket types
-                args, kwargs = cls._parse_qpel_as_args(qpel[i + 1:end])
+                args, kwargs = self._parse_qpel_as_args(qpel[i + 1:end])
                 i = end + 1
-                result = Call(body=result, args=args, kwargs=kwargs)
+                result = self.token_factory.make_call(body=result, args=args, kwargs=kwargs)
+            elif self.allow_spaces and char == " ":
+                i += 1
             else:
                 raise ValueError(f"Unexpected char: {char!r} in position {i}")
+
         return result
 
-    @classmethod
-    def _split_by_separator(cls, qpel: str, separator: str) -> list[str]:
+    @staticmethod
+    def _split_by_separator(qpel: str, separator: str) -> list[str]:
         result: list[str] = []
         last_position = 0
         brackets: list[str] = []
@@ -131,15 +133,14 @@ class QpelParser:
             result.append(qpel[last_position:])
         return result
 
-    @classmethod
-    def _parse_qpel_as_args(cls, qpel: str) -> tuple[list[Token], dict[str, Token]]:
+    def _parse_qpel_as_args(self, qpel: str) -> tuple[list[Token], dict[str, Token]]:
         args = []
         kwargs = {}
-        for part in cls._split_by_separator(qpel, separator=','):
+        for part in self._split_by_separator(qpel, separator=','):
             if not part:
                 raise ValueError("empty string in args")
-            sub_parts = cls._split_by_separator(part, separator='=')
-            val = cls.parse(sub_parts[-1])
+            sub_parts = self._split_by_separator(part, separator='=')
+            val = self.parse(sub_parts[-1])
             if len(sub_parts) == 1:
                 args.append(val)
             elif len(sub_parts) == 2:
